<?php

use Axepta\Payment\Notification;
use Axepta\Payment\Util;

/**
 * Class AxeptaNotificationModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_column_right
 * @property bool display_footer
 * @property bool display_header
 */
class AxeptaNotificationModuleFrontController extends ModuleFrontController
{

	/**
	 * Initialize controller.
	 *
	 * @see FrontController::init()
	 */
	public function init()
	{
		$this->display_column_left = false;
		$this->display_column_right = false;
		$this->display_footer = false;
		$this->display_header = false;
		parent::init();
	}

	/**
	 * Verify signature, json, create signature and change status if its okay
	 *
	 * @throws Exception
	 */
	public function process()
	{

		include_once(_PS_MODULE_DIR_ . 'axepta/libraries/payment-core/vendor/autoload.php');

		$notification = new Notification(
			Configuration::get('AXEPTA_SERVICE_ID'),
			Configuration::get('AXEPTA_SERVICE_KEY')
		);

		$resultCheckRequestNotification = $notification->checkRequest();

		if(is_int($resultCheckRequestNotification)) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				$resultCheckRequestNotification
			);
			exit();
		}

		$orderId = $resultCheckRequestNotification['payment']['orderId'];

		$order = new Order($orderId);

		$orderId = $order->id;

		if(!$orderId) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_ORDER_NOT_FOUND
			);
			exit();
		}

		$currentOrderState = $order->current_state;

		if($resultCheckRequestNotification['payment']['notifyTransactionData']['type'] === \Axepta\Payment\Api::TRANSACTION_TYPE_REFUND) {

			$statusBefore = '';
			$statusAfter = '';

			if($resultCheckRequestNotification['payment']['amountPaid'] === (int) $resultCheckRequestNotification['payment']['amountRefunded']) {

				$history = new OrderHistory();
				$history->id_order = $orderId;
				$history->changeIdOrderState(Configuration::get('PS_OS_REFUND'), $orderId);
				$history->addWithemail(true, []);

				$statusBefore = $currentOrderState;
				$statusAfter = Configuration::get('PS_OS_REFUND');
			}

			echo Notification::formatResponse(
				Notification::NS_OK,
				Notification::NC_OK,
				$statusBefore,
				$statusAfter
			);
			exit;
		}

		$currencyInfo = Currency::getCurrency($order->id_currency);

		if(!Notification::checkRequestAmount(
			$resultCheckRequestNotification,
			Util::convertAmountToFractional(round($order->total_paid, 2)),
			$currencyInfo['iso_code']
		)) {

			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_AMOUNT_NOT_MATCH
			);
			exit();
		}

		$transactionStatuses = Util::getTransactionStatuses();

		if(!isset($transactionStatuses[$resultCheckRequestNotification['payment']['status']])) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_UNHANDLED_STATUS
			);
			exit;
		}

		switch($resultCheckRequestNotification['payment']['status']) {
			case Notification::PS_SETTLED:
				$stateToChange = _PS_OS_PAYMENT_;
				break;
			case Notification::PS_REJECTED:
				$stateToChange = _PS_OS_ERROR_;
				break;
			default:
				echo Notification::formatResponse(
					Notification::NS_OK,
					Notification::NC_ORDER_STATUS_NOT_CHANGED,
					$currentOrderState
				);
				exit();
		}

		$history = new OrderHistory();
		$history->id_order = $orderId;
		$history->changeIdOrderState($stateToChange, $orderId);
		$history->addWithemail(true);

		$payments = $order->getOrderPaymentCollection()->getResults();

		if(count($payments) > 0) {
			$payments[0]->transaction_id = $resultCheckRequestNotification['payment']['id'];
			$payments[0]->update();
		}

		echo Notification::formatResponse(
			Notification::NS_OK,
			Notification::NC_OK,
			$currentOrderState,
			$stateToChange
		);
		exit;
	}
}
