<?php

use Axepta\Payment\Paywall;
use Axepta\Payment\Util;

/**
 * Class AxeptaPaymentModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_column_right
 */
class AxeptaPaymentModuleFrontController extends ModuleFrontController
{

    /**
     * Initialize controller
     *
     * @throws PrestaShopException
     * @see FrontController::init()
     */
    public function init()
    {
        $this->display_column_left = false;
        $this->display_column_right = false;
        parent::init();
    }

    /**
     * @throws PrestaShopException
     * @throws Exception
     */
    public function initContent()
    {
        parent::initContent();

        $paywall = new Paywall(
            Configuration::get('AXEPTA_SANDBOX')
                ? Util::ENVIRONMENT_SANDBOX
                : Util::ENVIRONMENT_PRODUCTION,
            Configuration::get('AXEPTA_SERVICE_KEY'),
            Configuration::get('AXEPTA_SERVICE_ID'),
            Configuration::get('AXEPTA_MERCHANT_ID')
        );

        $cart = $this->context->cart;

        if (!$cart->date_upd) {
            Tools::redirect('/');

            return;
        }

        $axepta = new Axepta();

        $this->context->smarty->assign([
            'form'                    => $paywall->buildOrderForm(
                $axepta->getDataForRequest($cart)
            ),
            'checkout_link'           => $this->context->link->getPageLink(Configuration::get('PS_ORDER_PROCESS_TYPE')
                ? 'order-opc'
                : 'order'),
            'text_return_to_checkout' => $this->module->l('Please wait, you will be returned to checkout.', 'payment'),
            'ga_key'                  => Configuration::get('AXEPTA_GA_KEY'),

            'loading_gif' => Media::getMediaPath(_PS_MODULE_DIR_ . $axepta->name . '/assets/img/loading.gif'),
        ]);

        $this->setTemplate('pay.tpl');
    }
}
